{ 2004-01-17 programmeerimisvistluse edasijudnute 3. lesande lahendus    }
{ lesanne: KIVI                                                            }
{ Autor: Kristo Tammeoja                                                    }
{ Testitud: FPC 1.0.10 / Win32                                              }

{ Dnaamiline programmeerimine O(n^2)                                       }

const
    MAX_N = 1000;
    SISFAIL = 'KIVI.SIS';
    VALFAIL = 'KIVI.VAL';

    TYHI = -1; // vrtuste jaoks, mida pole veel leitud
               // kuna muidu tegeleme lesandes mitte-negatiivsete arvudega,
               // siis sobib selleks suvaline negatiivne arv, niteks -1
var
    N, M : longint;

    // Kivikamaka andmed
    //
    // algoritmi lihtsuse huvides on kivikamaka vlimisteks
    // kihtideks pandud pehmed kihid (indeksiga 0 ja N)
    // kaalu ja puurimisajaga 0 (pseudokihid)
    //
    // kva kivimi kihtide massid
    mass_k : array[1..MAX_N] of longint;
    // pehme kivimi kihtide massid
    mass_p : array[0..MAX_N] of longint;
    // pehmesse kivimisse augu puurimiseks kuluv aeg
    aeg_p : array[0..MAX_N] of longint;

    // Arvutustes kasutatavad massiivid
    //
    // kivikamaka mass algusest kuni selle pehme kivimi kihini (kiht kaasa
    // arvatud); abiks kahe puuraugu vahele jva tki massi arvutamisel
    mass_p_kihini : array[0..MAX_N] of longint;

    // Vastuse kirjutamisel kasutatavad muutujad
    //
    // puuraukude arv
    vastus_cnt : longint;
    // vastusesse kirjutatavad arvud vastupidises jrjekorras
    vastus_desc : array[1..MAX_N] of longint;

    // Lahendamisel kasutatavad muutujad
    //
    // massiivi i-s element nitab minimaalset aega, mis kulub
    // kivikamaka alguse sobiva suurusega tkkideks puurimiseks juhul,
    // kui viimane auk puuriti i-ndasse pehme kivimi kihti...
    min_aeg : array[0..MAX_N] of longint;
    // ...ja (ks vimalik) eelmine kiht, kuhu tuleb sellisel juhul auk
    // puurida (selle massiivi abil saab lpuks kostrueerida vastusesse
    // sobivad kihtide numbrid)
    min_eelmine : array[0..MAX_N] of longint;


// Kivitki mass, mis jb pehmete kihtide a ja b vahele (a <= b)
function kivityki_mass (a, b : longint) : longint;
begin
    if a = b then
        kivityki_mass := 0
    else
        kivityki_mass := mass_p_kihini[b] - mass_p_kihini[a] -
                         mass_p[b];
end; //kivityki_mass


procedure lahenda;
var
    puurauk : longint;
    parim_eelmine : longint;
    l : longint;
begin
    // esimene pseudokiht on nn baaslesanne,
    // mille vastust on lihtne arvutada
    min_aeg[0] := 0;
    min_eelmine[0] := TYHI;

    for puurauk := 1 to N do begin
        // leiame, kus pidi olema eelmine puurauk, et praegu
        // vaadeldava kihi ja eelmise puuraugu vahele jks kivitkk,
        // mille mass on viksem kui kriitiline mass, ja
        // kivi algus kuni praeguse kihini oleks tkeldatud minimaalse
        // ajaga (tingimusel, et praegusesse kihti puuritakse ka auk).
        parim_eelmine := puurauk-1;
        for l := puurauk-2 downto 0 do begin
            // kui praeguse ja eelmise puuraugu vahel olev osa
            // jb juba liiga paksuks.
            if kivityki_mass(l, puurauk) > M then break;

            // kui kiht l sobib paremini eelmiseks
            if min_aeg[l] < min_aeg[parim_eelmine] then
                parim_eelmine := l;
        end;

        // Nd on muutujas parim_eelmine kirjas sobivaim kandidaat
        // eelmis puuraugu kohale.

        // kirjutame tulemuse vaadeldava puuraugu jaoks
        min_aeg[puurauk] := min_aeg[parim_eelmine] + aeg_p[puurauk];
        min_eelmine[puurauk] := parim_eelmine;
    end; //for puurauk
end; //lahenda


procedure loe_andmed;
var
    l : longint;
    sfail : text;
begin
    assign (sfail, SISFAIL); reset (sfail);

    readln (sfail, N, M);

    // kva kivimi kihid
    for l := 1 to N do
        readln (sfail, mass_k[l]);

    // pehme kivimi kihid
    for l := 1 to N-1 do
        readln (sfail, mass_p[l], aeg_p[l]);

    // pseudokihid
    mass_p[0] := 0;
    mass_p[N] := 0;
    aeg_p[0] := 0;
    aeg_p[N] := 0;

    // abimassiivid
    mass_p_kihini[0] := mass_p[0];
    for l := 1 to N do
        // kivikamaka mass algusest i-nda pehme kihini on vrdne
        // kivikamaka massiga (i-1)-se pehme kihini..
        mass_p_kihini[l] := mass_p_kihini[l-1] +
                            // .. ja lisanduvate kihtide masside summaga
                            mass_k[l] + mass_p[l];


    close (sfail);
end; //loe_andmed


procedure kirjuta_vastus;
var
    vfail : text;
    l : longint;
    puurauk : longint;
begin
    assign (vfail, VALFAIL); rewrite (vfail);

    // Minimaalne puurimiseks kuluv koguaeg on kirjas min_aeg[N]

    // Konstrueerime nd massiivi min_eelmine abil vastuse.
    // Viimane, pseudoauk puuriti N-ndasse pehme kivimi kihti
    puurauk := N;
    vastus_cnt := 0; // esialgu pole hegi augu andmeid

    // liigume puurauke mda kivikamaka alguse poole
    while min_eelmine[puurauk] > 0 do begin
        // kui puurauk-ist eelmine puuritav kiht pole
        // veel esimene pseudokiht
        puurauk := min_eelmine[puurauk];

        // muutujas puurauk on kirjas uus puuritud kihi number
        vastus_cnt := vastus_cnt + 1;
        vastus_desc[vastus_cnt] := puurauk;
    end;

    // kirjutame vastuse

    // puuraukude arv
    writeln (vfail, vastus_cnt);
    // kihtide numbrid kasvavas jrjekorras
    for l := vastus_cnt downto 1 do
        writeln (vfail, vastus_desc[l]);

    close (vfail);
end; //kirjuta_vastus


begin
    loe_andmed;
    lahenda;
    kirjuta_vastus;
end.
