{ 2004-01-17 programmeerimisvistluse edasijudnute 3. lesande lahendus    }
{ lesanne: KIVI                                                            }
{ Autor: Kristo Tammeoja                                                    }
{ Testitud: FPC 1.0.10 / Win32                                              }

{ Dnaamiline programmeerimine, likude puuga O(n*log(n))                   }

const
    MAX_N = 1000;
    SISFAIL = 'KIVI.SIS';
    VALFAIL = 'KIVI.VAL';

    TYHI = -1; // vrtuste jaoks, mida pole veel leitud
               // kuna muidu tegeleme lesandes mitte-negatiivsete arvudega,
               // siis sobib selleks suvaline negatiivne arv, niteks -1
var
    N, M : longint;

    // Kivikamaka andmed
    //
    // algoritmi lihtsuse huvides on kivikamaka vlimisteks
    // kihtideks pandud pehmed kihid (indeksiga 0 ja N)
    // kaalu ja puurimisajaga 0 (pseudokihid)
    //
    // kva kivimi kihtide massid
    mass_k : array[1..MAX_N] of longint;
    // pehme kivimi kihtide massid
    mass_p : array[0..MAX_N] of longint;
    // pehmesse kivimisse augu puurimiseks kuluv aeg
    aeg_p : array[0..MAX_N] of longint;

    // Arvutustes kasutatavad massiivid
    //
    // kivikamaka mass algusest kuni selle pehme kivimi kihini (kiht kaasa
    // arvatud); abiks kahe puuraugu vahele jva tki massi arvutamisel
    mass_p_kihini : array[0..MAX_N] of longint;
    // Massiivi pakitud kahendpuu. Iga tipp sisaldab sellise kihi numbrit
    // mingis vahemikus [a, b], millel min_aeg on vhim.
    // Infot min_aeg vrtuste kohta lisatakse massiivi jooksvalt.
    min_puu : array[1..4*(MAX_N+1)] of longint;

    // Vastuse kirjutamisel kasutatavad muutujad
    //
    // puuraukude arv
    vastus_cnt : longint;
    // vastusesse kirjutatavad arvud vastupidises jrjekorras
    vastus_desc : array[1..MAX_N] of longint;

    // Lahendamisel kasutatavad muutujad
    //
    // massiivi i-s element nitab minimaalset aega, mis kulub
    // kivikamaka alguse sobiva suurusega tkkideks puurimiseks juhul,
    // kui viimane auk puuriti i-ndasse pehme kivimi kihti...
    min_aeg : array[0..MAX_N] of longint;
    // ...ja (ks vimalik) eelmine kiht, kuhu tuleb sellisel juhul auk
    // puurida (selle massiivi abil saab lpuks kostrueerida vastusesse
    // sobivad kihtide numbrid)
    min_eelmine : array[0..MAX_N] of longint;


// Kivitki mass, mis jb pehmete kihtide a ja b vahele (a <= b)
function kivityki_mass (a, b : longint) : longint;
begin
    if a = b then
        kivityki_mass := 0
    else
        kivityki_mass := mass_p_kihini[b] - mass_p_kihini[a] -
                         mass_p[b];
end; //kivityki_mass


// Leiab esimese (vhima indeksiga) kihi vahemikus [0, ..., puurauk-1]
// selliselt, et tema ja kihi puurauk vahele jv kivi tki mass oleks
// veel kriitilisest massist viksem vi vrdne.
// lesande eelduste phjal teame, et selline kiht on olemas.
function leia_esimene_sobiv (puurauk : longint) : longint;
var
    // vahemiku otspunktid, kus meid huvitav vastus on
    a, b : longint;
    // abimuutuja
    c : longint;
begin
    // Esialgne hinnang vastuse paiknemise kohta
    a := 0;
    b := puurauk-1;

    // Me teame, et otsitav vastus on vahemikus [a, b].
    // Kahandame seda vahemikku kui ta on pikem kui 1 element
    while a < b do begin
        // Jagame vahemiku [a, b] kaheks enam-vhem vrdseks
        // osaks [a, c] ja [c+1, b].
        c := (a+b) div 2; // (a+b)/2 alumine tisosa
                          // Selline keskmise elemendi valik tagab, et
                          // mlemad tekkivad alamvahemikud oleksid
                          // viksemad kui algne.

        if kivityki_mass (c, puurauk) > M then
            // Kui isegi kihtide c ja puurauk vahele jva kivitki mass
            // on kriitilisest massist suurem, siis on mtet vastust otsida
            // vaid vahemikust [c+1, b]
            a := c+1
        else
            // Kuid kui kihtide c ja puurauk vahele jva kivitki mass
            // pole kriitilisest massist suurem, siis on mtet otsida
            // vhimat sobivat kihi numbrit vaid vahemikust [a, c]
            b := c;
    end;

    // a=b
    leia_esimene_sobiv := a;
end; //leia_esimene_sobiv


//============ Puuga ttamiseks vajalikud meetodid =================

// Tagastab kahest pehmest kihist k ja l selle, millel
// min_aeg on viksem. Kui ks parameetritest on TYHI,
// siis tagastatakse teise parameetri vrtus.
function min_ajaga_kiht (k, l : longint) : longint;
begin
    if k = TYHI then
        min_ajaga_kiht := l
    else if l = TYHI then
        min_ajaga_kiht := k
    else if min_aeg[k] < min_aeg[l] then
        min_ajaga_kiht := k
    else
        min_ajaga_kiht := l;
end; //min_ajaga_kiht


// Leiab sarnaselt meetodile leia_vahim_vahemikus pehme kihi
// vahemikus [a, b], millel min_aeg oleks vhim.
// i on kahendpuu tipu, mis thistab liku [k, l], indeks
// massiivis min_puu.
// Eeldused:
//  * Kui k=0 ja l=N, siis i=1.
//  * Peab kehtima seos k <= a <= b <= l.
function leia_vahim_puus (i : longint;
                          k, l : longint;
                          a, b : longint) : longint;
var
    m : longint;
begin
    // Kui me saame vastuse lugeda puust
    if (k=a) and (l=b) then begin
        leia_vahim_puus := min_puu[i];
        exit;
    end;

    // Jagame vahemiku [k, l] kaheks osaks [k, m] ja [m+1, l]
    // (tippude indeksitega vastavalt 2*i ja 2*i+1)
    // ja lahendame lesande nende kummagi jaoks eraldi...
    m := (k+l) div 2; // vaata selgitust meetodis leia_esimene_sobiv

    // On kolm vimalust, kuidas lik [a, b] vib likude
    // [k, m] ja [m+1, l] suhtes asuda:
    if b <= m then
        // [k, ......., m][m+1, ......, l]
        //   [a, ..., b]
        leia_vahim_puus := leia_vahim_puus (2*i, k, m, a, b)
    else if m < a then
        // [k, ......., m][m+1, ......, l]
        //                  [a, ..., b]
        leia_vahim_puus := leia_vahim_puus (2*i+1, m+1, l, a, b)
    else
        // [k, ......., m][m+1, ......, l]
        //          [a, ..., b]
        leia_vahim_puus := min_ajaga_kiht (
                               leia_vahim_puus (2*i,   k,   m, a,   m),
                               leia_vahim_puus (2*i+1, m+1, l, m+1, b)
                                          );
end; //leia_vahim_puus


// Leiab (vimalik, et he mitmest) pehme kihi vahemikust [a, b]
// (tingimusel, et a <= b), millel min_aeg oleks vhim.
function leia_vahim_vahemikus (a, b : longint) : longint;
begin
    leia_vahim_vahemikus := leia_vahim_puus (1, 0, N, a, b);
end; //leia_vahim_vahemikus


// Lisab puusse parameetriga kiht ette antud kihi min_aeg vrtuse
// i on kahendpuu tipu, mis thistab liku [k, l], indeks
// massiivis min_puu.
procedure lisa_min_aeg_puusse (i : longint;
                               k, l : longint;
                               kiht : longint);
var
    m : longint;
begin
    if (k = kiht) and (kiht = l) then begin
        // Kui me tegeleme hest elemendist koosneva liguga,
        // siis on lihtne
        min_puu[i] := kiht;
    end else begin
        // Kui lik [k, l] on pikem, kui ks element, siis on ka lihtne
        // Jagame ligu kaheks osaks, et teada saada alamtippude poolt
        // kaetavad ligud [k, m] ning [m+1, l]...
        m := (k+l) div 2;

        // ...ja kutsume meetodi sama kihi jaoks uuesti vlja, kuid
        // sedapuhku he alamtipu jaoks. Prast seda tuleb uuendada
        // ka tipu juures meeles peetava kihi numbrit.
        if kiht <= m then
            lisa_min_aeg_puusse (2*i, k, m, kiht)
        else
            lisa_min_aeg_puusse (2*i+1, m+1, l, kiht);

        // Nd tuleb uuendada ka tipu enda juures meeles peetava
        // kihi numbrit. Ligus [k, l] olev kiht, mille min_aeg
        // vrtus on viksem, asub ka hes kahest ligust [k, m]
        // ja [m+1, l]. Seega
        min_puu[i] := min_ajaga_kiht (min_puu[2*i],    //lik [k,   m]
                                      min_puu[2*i+1]); //lik [m+1, l]
    end; // kui k < l
end; //lisa_min_aeg_puusse


// Lisab puusse parameetriga ette antud kihi min_aeg vrtuse
procedure lisa_min_aeg (kiht : longint);
begin
    lisa_min_aeg_puusse (1, 0, N, kiht);
end; //lisa_min_aeg

//========== Puuga ttamiseks vajalikud meetodid (lpp) ===============


procedure lahenda;
var
    puurauk : longint;
    parim_eelmine : longint;
    esimene_sobiv : longint;
begin
    // esimene pseudokiht on nn baaslesanne,
    // mille vastust on lihtne arvutada
    min_aeg[0] := 0;
    min_eelmine[0] := TYHI;
    lisa_min_aeg (0);

    for puurauk := 1 to N do begin
        // leiame, kus pidi olema eelmine puurauk, et praegu
        // vaadeldava kihi ja eelmise puuraugu vahele jks kivitkk,
        // mille mass on viksem kui kriitiline mass, ja
        // kivi algus kuni praeguse kihini oleks tkeldatud minimaalse
        // ajaga (tingimusel, et praegusesse kihti puuritakse ka auk).

        // Leiame kige varasema kihi, mis vib olla praegusele puuraugule
        // eelnevaks lhatavaks kihiks.
        // Tagastatav kihi number on vahemikus [0, ..., puurauk-1].
        esimene_sobiv := leia_esimene_sobiv (puurauk);

        // Leiame kihi vahemikus [esimene_sobiv, ..., puurauk-1],
        // millel min_aeg on vhim.
        parim_eelmine := leia_vahim_vahemikus (esimene_sobiv, puurauk-1);

        // Nd on muutujas parim_eelmine kirjas sobivaim kandidaat
        // eelmis puuraugu kohale.

        // kirjutame tulemuse vaadeldava puuraugu jaoks
        min_aeg[puurauk] := min_aeg[parim_eelmine] + aeg_p[puurauk];
        min_eelmine[puurauk] := parim_eelmine;
        lisa_min_aeg (puurauk);
    end; //for puurauk
end; //lahenda


procedure loe_andmed;
var
    l : longint;
    sfail : text;
begin
    assign (sfail, SISFAIL); reset (sfail);

    readln (sfail, N, M);

    // kva kivimi kihid
    for l := 1 to N do
        readln (sfail, mass_k[l]);

    // pehme kivimi kihid
    for l := 1 to N-1 do
        readln (sfail, mass_p[l], aeg_p[l]);

    // pseudokihid
    mass_p[0] := 0;
    mass_p[N] := 0;
    aeg_p[0] := 0;
    aeg_p[N] := 0;

    // abimassiivid
    mass_p_kihini[0] := mass_p[0];
    for l := 1 to N do
        // kivikamaka mass algusest i-nda pehme kihini on vrdne
        // kivikamaka massiga (i-1)-se pehme kihini..
        mass_p_kihini[l] := mass_p_kihini[l-1] +
                            // .. ja lisanduvate kihtide masside summaga
                            mass_k[l] + mass_p[l];

    // Puus mrgime kikide tippud vrtuseks TYHI, sest me ei tea
    // veel hegi ligu kohta, millise selles ligus oleva kihi
    // korral on min_aeg vhim.
    for l := 1 to 4*(N+1) do
        min_puu[l] := TYHI;

    close (sfail);
end; //loe_andmed


procedure kirjuta_vastus;
var
    vfail : text;
    l : longint;
    puurauk : longint;
begin
    assign (vfail, VALFAIL); rewrite (vfail);

    // Minimaalne puurimiseks kuluv koguaeg on kirjas min_aeg[N]

    // Konstrueerime nd massiivi min_eelmine abil vastuse.
    // Viimane, pseudoauk puuriti N-ndasse pehme kivimi kihti
    puurauk := N;
    vastus_cnt := 0; // esialgu pole hegi augu andmeid

    // liigume puurauke mda kivikamaka alguse poole
    while min_eelmine[puurauk] > 0 do begin
        // kui puurauk-ist eelmine puuritav kiht pole
        // veel esimene pseudokiht
        puurauk := min_eelmine[puurauk];

        // muutujas puurauk on kirjas uus puuritud kihi number
        vastus_cnt := vastus_cnt + 1;
        vastus_desc[vastus_cnt] := puurauk;
    end;

    // kirjutame vastuse

    // puuraukude arv
    writeln (vfail, vastus_cnt);
    // kihtide numbrid kasvavas jrjekorras
    for l := vastus_cnt downto 1 do
        writeln (vfail, vastus_desc[l]);

    close (vfail);
end; //kirjuta_vastus


begin
    loe_andmed;
    lahenda;
    kirjuta_vastus;
end.
