program kontroll;

{$I+}

const
    MAX_N = 1000;

    TYHI = -1; // vrtuste jaoks, mida pole veel leitud
               // kuna muidu tegeleme lesandes mitte-negatiivsete arvudega,
               // siis sobib selleks suvaline negatiivne arv, niteks -1
var
    N, M : longint;

    // Kivikamaka andmed
    //
    // algoritmi lihtsuse huvides on kivikamaka vlimisteks
    // kihtideks pandud pehmed kihid (indeksiga 0 ja N)
    // kaalu ja puurimisajaga 0 (pseudokihid)
    //
    // kva kivimi kihtide massid
    mass_k : array[1..MAX_N] of longint;
    // pehme kivimi kihtide massid
    mass_p : array[0..MAX_N] of longint;
    // pehmesse kivimisse augu puurimiseks kuluv aeg
    aeg_p : array[0..MAX_N] of longint;

    // Arvutustes kasutatavad massiivid
    //
    // kivikamaka mass algusest kuni selle pehme kivimi kihini (kiht kaasa
    // arvatud); abiks kahe puuraugu vahele jva tki massi arvutamisel
    mass_p_kihini : array[0..MAX_N] of longint;

    // Lahendamisel kasutatavad muutujad
    //
    // massiivi i-s element nitab minimaalset aega, mis kulub
    // kivikamaka alguse sobiva suurusega tkkideks puurimiseks juhul,
    // kui viimane auk puuriti i-ndasse pehme kivimi kihti...
    min_aeg : array[0..MAX_N] of longint;

    // Parim aeg, millega on vimalik kivikamakas sobiva suurusega
    // tkkideks teha.
    parim_aeg : longint;


// Kivitki mass, mis jb pehmete kihtide a ja b vahele (a <= b)
function kivityki_mass (a, b : longint) : longint;
begin
    if a = b then
        kivityki_mass := 0
    else
        kivityki_mass := mass_p_kihini[b] - mass_p_kihini[a] -
                         mass_p[b];
end; //kivityki_mass


procedure lahenda;
var
    puurauk : longint;
    parim_eelmine : longint;
    l : longint;
begin
    // esimene pseudokiht on nn baaslesanne,
    // mille vastust on lihtne arvutada
    min_aeg[0] := 0;

    for puurauk := 1 to N do begin
        // leiame, kus pidi olema eelmine puurauk, et praegu
        // vaadeldava kihi ja eelmise puuraugu vahele jks kivitkk,
        // mille mass on viksem kui kriitiline mass, ja
        // kivi algus kuni praeguse kihini oleks tkeldatud minimaalse
        // ajaga (tingimusel, et praegusesse kihti puuritakse ka auk).
        parim_eelmine := puurauk-1;
        for l := puurauk-2 downto 0 do begin
            // kui praeguse ja eelmise puuraugu vahel olev osa
            // jb juba liiga paksuks.
            if kivityki_mass(l, puurauk) > M then break;

            // kui kiht l sobib paremini eelmiseks
            if min_aeg[l] < min_aeg[parim_eelmine] then
                parim_eelmine := l;
        end;

        // Nd on muutujas parim_eelmine kirjas sobivaim kandidaat
        // eelmis puuraugu kohale.

        // kirjutame tulemuse vaadeldava puuraugu jaoks
        min_aeg[puurauk] := min_aeg[parim_eelmine] + aeg_p[puurauk];
    end; //for puurauk

    parim_aeg := min_aeg[N];
end; //lahenda


procedure loe_andmed;
var
    l : longint;
    sfail : text;
begin
    assign (sfail, paramstr(1)); reset (sfail);

    readln (sfail, N, M);

    // kva kivimi kihid
    for l := 1 to N do
        readln (sfail, mass_k[l]);

    // pehme kivimi kihid
    for l := 1 to N-1 do
        readln (sfail, mass_p[l], aeg_p[l]);

    // pseudokihid
    mass_p[0] := 0;
    mass_p[N] := 0;
    aeg_p[0] := 0;
    aeg_p[N] := 0;

    // abimassiivid
    mass_p_kihini[0] := mass_p[0];
    for l := 1 to N do
        // kivikamaka mass algusest i-nda pehme kihini on vrdne
        // kivikamaka massiga (i-1)-se pehme kihini..
        mass_p_kihini[l] := mass_p_kihini[l-1] +
                            // .. ja lisanduvate kihtide masside summaga
                            mass_k[l] + mass_p[l];


    close (sfail);
end; //loe_andmed


procedure kontrolli_vastust;
var
    vfail : text;
    // lahenduse andmed
    lah_aeg : longint;
    lah_cnt : longint;
    eelmine_auk : longint;
    puurauk : longint;
    l : longint;
begin
    {$I-}
    assign (vfail, paramstr(2)); reset (vfail);
    if IOResult <> 0 then begin
        writeln ('Viga vljundfaili avamisel.');
        exit;
    end;

    readln (vfail, lah_cnt);
    if IOResult <> 0 then begin
        writeln ('Viga puuraukude arvu leidmisel.');
        exit;
    end;
    if not ((0<=lah_cnt) and (lah_cnt<=N-1)) then begin
        writeln ('Viga: puuraukude arv ei sobi.');
        exit;
    end;

    lah_aeg := 0;
    eelmine_auk := 0;

    for l := 1 to lah_cnt do begin
        readln (vfail, puurauk);
        if IOResult <> 0 then begin
            writeln ('Viga ', l, '. puuraugu numbri leidmisel');
            exit;
        end;
        if not ((1<=puurauk) and (puurauk<=N-1)) then begin
            writeln ('Viga: ', l, '. puuraugu number ei sobi');
            exit;
        end;
        if eelmine_auk >= puurauk then begin
            writeln ('Viga: ', l, '. puuraugu number pole eelmisest suurem');
            exit;
        end;
        if kivityki_mass (eelmine_auk, puurauk) > M then begin
            writeln ('Viga: ', l, '. puuraugule eelnev kivitkk liiga suur');
            exit;
        end;

        lah_aeg := lah_aeg + aeg_p[puurauk];
        eelmine_auk := puurauk;
    end;

    if kivityki_mass (eelmine_auk, N) > M then begin
        writeln ('Viimase kivitki mass liiga suur.');
        exit;
    end;
    if lah_aeg > parim_aeg then begin
        writeln ('Puurimisaeg pole optimaalne.');
        exit;
    end;
    if lah_aeg < parim_aeg then begin
        writeln ('!!!!! Korrektsel lahendusel on nidislahendusest parem aeg !!!!!');
        exit;
    end;

    writeln ('OK');

    close (vfail);
    {$I+}
end; //kirjuta_vastus


begin
    if (paramcount <> 2) then begin
        writeln('Sntaks on: ', paramstr(0), ' sisendfail vljundfail');
        exit;
    end;

    loe_andmed;
    lahenda;
    kontrolli_vastust;
end.
